package aliyun

import (
	"io"
	"mime/multipart"
	"xmall/internal/config"
	"xmall/pkg/xerrors/xerror"

	"github.com/aliyun/aliyun-oss-go-sdk/oss"
)

type AliyunOSS struct{}

var OSS = &AliyunOSS{}

func NewClient() (bucket *oss.Bucket, err error) {
	endpoint := ""
	accessKeyId := ""
	accessKeySecret := ""
	region := ""
	bucketName := ""

	config.Read(func(c *config.Config) {
		endpoint = c.Extend.AliOss.Endpoint
		accessKeyId = c.Extend.AliOss.AccessKeyId
		accessKeySecret = c.Extend.AliOss.AccessKeySecret
		region = c.Extend.AliOss.Region
		bucketName = c.Extend.AliOss.BucketName
	})
	client, err := oss.New(endpoint,
		accessKeyId,
		accessKeySecret,
		oss.AuthVersion(oss.AuthV4),
		oss.Region(region))
	if err != nil {
		return nil, xerror.New(err.Error())
	}
	bucket, err = client.Bucket(bucketName)
	return
}

func NewBucket() (bucket *oss.Bucket, err error) {
	endpoint := ""
	accessKeyId := ""
	accessKeySecret := ""
	bucketName := ""

	config.Read(func(c *config.Config) {
		endpoint = c.Extend.AliOss.Endpoint
		accessKeyId = c.Extend.AliOss.AccessKeyId
		accessKeySecret = c.Extend.AliOss.AccessKeySecret
		bucketName = c.Extend.AliOss.BucketName
	})
	client, err := oss.New(endpoint, accessKeyId, accessKeySecret)
	if err != nil {
		return
	}
	bucket, err = client.Bucket(bucketName)
	return
}

func (*AliyunOSS) UploadFile(reader io.Reader, name string) (string, string, error) {
	basePath := ""
	bucketUrl := ""

	config.Read(func(c *config.Config) {
		basePath = c.Extend.AliOss.BasePath
		bucketUrl = c.Extend.AliOss.BucketUrl
	})

	bucket, err := NewClient()
	if err != nil {
		return "", "", err
	}
	fileTmpPath := basePath + "/" + name
	err = bucket.PutObject(fileTmpPath, reader)
	if err != nil {
		return "", "", xerror.New(err.Error())
	}
	return bucketUrl + "/" + fileTmpPath, fileTmpPath, nil
}

type Attachment struct {
	File        io.Reader
	Name        string
	AbsUrl      string
	RelativeUrl string
}

func (x Attachment) NewFromMultipartFileHeaders(files []*multipart.FileHeader) ([]*Attachment, error) {
	if len(files) == 0 {
		return nil, nil
	}
	attachments := make([]*Attachment, len(files))
	for i := range files {
		reader, err := files[i].Open()
		if err != nil {
			return nil, xerror.New(err.Error())
		}
		attachment := &Attachment{
			File: reader,
			Name: files[i].Filename,
		}
		attachments = append(attachments, attachment)
	}
	return attachments, nil
}

func (x *AliyunOSS) UploadMultipartFileHeaders(files []*multipart.FileHeader) ([]*Attachment, error) {
	if len(files) == 0 {
		return nil, nil
	}

	attachments, err := Attachment{}.NewFromMultipartFileHeaders(files)
	if err != nil {
		return nil, err
	}

	err = x.UploadFiles(&attachments)
	if err != nil {
		return nil, err
	}

	return attachments, nil
}

func (*AliyunOSS) UploadFiles(files *[]*Attachment) error {
	if len(*files) == 0 {
		return nil
	}
	basePath := ""
	bucketUrl := ""

	config.Read(func(c *config.Config) {
		basePath = c.Extend.AliOss.BasePath
		bucketUrl = c.Extend.AliOss.BucketUrl
	})

	bucket, err := NewClient()
	if err != nil {
		return xerror.New(err.Error())
	}
	for i, file := range *files {
		relativeUri := basePath + "/" + file.Name
		err = bucket.PutObject(relativeUri, file.File)
		if err != nil {
			return xerror.New(err.Error())
		}
		(*files)[i].AbsUrl = bucketUrl + "/" + relativeUri
		(*files)[i].RelativeUrl = relativeUri
	}
	return nil
}
